"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var commands_1 = require("@codemirror/commands");
var language_1 = require("@codemirror/language");
var view_1 = require("@codemirror/view");
var auto_1 = __importDefault(require("chart.js/auto"));
var debounce_1 = __importDefault(require("debounce"));
var manate_1 = require("manate");
var react_1 = require("manate/react");
var markplus_engine_1 = __importDefault(require("markplus-engine"));
var mermaid_esm_mjs_1 = __importDefault(require("mermaid/dist/mermaid.esm.mjs"));
var react_2 = __importStar(require("react"));
var sync_scroll_1 = require("../sync-scroll");
var Editor = function (props) {
    var store = props.store;
    var editorDiv = (0, react_2.useRef)(null);
    (0, react_2.useEffect)(function () {
        var contentChangeListener = view_1.EditorView.updateListener.of(function (update) {
            if (update.docChanged) {
                store.onChange(store.editor.state.doc.toString());
                lazyChange();
            }
        });
        // Define the custom key binding
        var customKeymap = view_1.keymap.of([
            {
                key: 'Tab',
                run: commands_1.indentMore,
            },
            {
                key: 'Shift-Tab',
                run: commands_1.indentLess,
            },
            {
                key: 'Mod-s',
                run: function () { return true; },
            },
            {
                key: 'Mod-b',
                run: function () {
                    var _a;
                    (_a = document
                        .querySelector("#".concat(store.uid, " .toolbar i.fa-bold"))) === null || _a === void 0 ? void 0 : _a.click();
                    return true;
                },
            },
        ]);
        var cm = new view_1.EditorView({
            extensions: [
                view_1.EditorView.lineWrapping,
                (0, view_1.highlightActiveLine)(),
                (0, commands_1.history)(),
                view_1.keymap.of(__spreadArray(__spreadArray([], commands_1.defaultKeymap, true), commands_1.historyKeymap, true)), // todo: type casting is unnecessary in the future
                (0, language_1.syntaxHighlighting)(language_1.defaultHighlightStyle),
                customKeymap,
                contentChangeListener,
            ],
            parent: editorDiv.current,
        });
        // auto focus after change text
        var dispatch = cm.dispatch.bind(cm);
        cm.dispatch = function (tr) {
            dispatch(tr);
            if (tr.changes && tr.changes.insert) {
                cm.focus();
            }
        };
        store.editor = (0, manate_1.exclude)(cm);
        var _a = (0, sync_scroll_1.generateScrollMethods)(store), syncEditor = _a.syncEditor, syncPreview = _a.syncPreview;
        var handleEditorScroll = function () {
            syncPreview();
        };
        store.editor.scrollDOM.addEventListener('scroll', handleEditorScroll);
        var handlePreviewScroll = function () {
            syncEditor();
        };
        var rightPanel = document.querySelector("#".concat(store.uid, " .right-panel"));
        rightPanel.addEventListener('scroll', handlePreviewScroll);
        // whenever user changes markdown
        mermaid_esm_mjs_1.default.initialize({ startOnLoad: false, theme: 'neutral' });
        var lazyChange = (0, debounce_1.default)(function () {
            var markdown = store.editor.state.doc.toString();
            document.querySelector("#".concat(store.uid, " .right-panel .preview")).innerHTML = markplus_engine_1.default.render(markdown);
            // Chart.js
            document
                .querySelectorAll("#".concat(store.uid, " .right-panel .preview .chartjs"))
                .forEach(function (element) {
                try {
                    new auto_1.default(element, JSON.parse(element.textContent));
                }
                catch (e) {
                    element.outerHTML = "<pre>Chart.js complains: \"".concat(e, "\"</pre>");
                }
            });
            // mermaid
            mermaid_esm_mjs_1.default.run({
                nodes: document.querySelectorAll("#".concat(store.uid, " .right-panel .preview pre.mermaid")),
            });
            // todo: remove this? user should generate html instead
            setTimeout(function () {
                store.onPreviewChange(document.querySelector('.right-panel .preview').outerHTML);
            }, 1000); // because it takes time for mermaid and chart.js to render
        }, 256);
        return function () {
            store.editor.scrollDOM.removeEventListener('scroll', handleEditorScroll);
            store.editor.destroy();
            rightPanel.removeEventListener('scroll', handlePreviewScroll);
        };
    }, [store]);
    return react_2.default.createElement("div", { className: "editor", ref: editorDiv });
};
exports.default = (0, react_1.auto)(Editor);
//# sourceMappingURL=editor.js.map