"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateScrollMethods = void 0;
var view_1 = require("@codemirror/view");
var debounce_1 = __importDefault(require("debounce"));
var utils_1 = require("./utils");
var generateScrollMethods = function (store) {
    var scrollingSide = null;
    var timeoutHandle = null;
    var scrollSide = function (side, howToScroll) {
        if (scrollingSide !== null && scrollingSide !== side) {
            return; // the other side hasn't finished scrolling
        }
        scrollingSide = side;
        clearTimeout(timeoutHandle);
        timeoutHandle = setTimeout(function () {
            scrollingSide = null;
        }, 512);
        howToScroll();
    };
    var scrollEditor = function (targetLineNumber) {
        scrollSide('left', function () {
            (0, utils_1.animate)(function (lineNumber) {
                return store.editor.dispatch({
                    effects: view_1.EditorView.scrollIntoView(store.editor.state.doc.line(lineNumber).from, { y: 'start' }),
                });
            }, store.editor.state.doc.lineAt(store.editor.lineBlockAtHeight(store.editor.scrollDOM.scrollTop).from).number, targetLineNumber, 128);
        });
    };
    var scrollPreview = function (scrollTop) {
        var rightPanel = document.querySelector("#".concat(store.uid, " .right-panel"));
        scrollSide('right', function () {
            (0, utils_1.animate)(function (i) { return (rightPanel.scrollTop = i); }, rightPanel.scrollTop, scrollTop, 128);
        });
    };
    var getEditorScroll = function () {
        var lineMarkers = document.querySelectorAll("#".concat(store.uid, " .right-panel .preview > [data-sl]"));
        var lines = [];
        lineMarkers.forEach(function (element) {
            lines.push(parseInt(element.dataset.sl, 10));
        });
        var currentLine = store.editor.state.doc.lineAt(store.editor.lineBlockAtHeight(store.editor.scrollDOM.scrollTop).from).number;
        var lastMarker;
        var nextMarker;
        for (var i = 0; i < lines.length; i++) {
            if (lines[i] < currentLine) {
                lastMarker = lines[i];
            }
            else {
                nextMarker = lines[i];
                break;
            }
        }
        var percentage = 0;
        if (lastMarker && nextMarker && lastMarker !== nextMarker) {
            percentage = (currentLine - lastMarker) / (nextMarker - lastMarker);
        }
        // returns two neighboring markers' lines, and current scroll percentage between two markers
        var r = { lastMarker: lastMarker, nextMarker: nextMarker, percentage: percentage };
        return r;
    };
    var setPreviewScroll = function (editorScroll) {
        var lastPosition = 0;
        var nextPosition = document.querySelector("#".concat(store.uid, " .right-panel .preview"))
            .offsetHeight -
            document.querySelector("#".concat(store.uid, " .right-panel"))
                .offsetHeight; // maximum scroll
        if (editorScroll.lastMarker) {
            var lastMarkerElement = document.querySelector("#".concat(store.uid, " .right-panel .preview > [data-sl=\"").concat(editorScroll.lastMarker, "\"]"));
            if (lastMarkerElement) {
                lastPosition = lastMarkerElement.offsetTop;
            }
        }
        if (editorScroll.nextMarker) {
            var nextMarkerElement = document.querySelector("#".concat(store.uid, " .right-panel .preview > [data-sl=\"").concat(editorScroll.nextMarker, "\"]"));
            if (nextMarkerElement) {
                nextPosition = nextMarkerElement.offsetTop;
            }
        }
        var scrollTop = lastPosition + (nextPosition - lastPosition) * editorScroll.percentage; // right scroll according to left percentage
        scrollPreview(scrollTop);
    };
    var getPreviewScroll = function () {
        var rightPanel = document.querySelector("#".concat(store.uid, " .right-panel"));
        var preview = document.querySelector("#".concat(store.uid, " .right-panel .preview"));
        var scroll = rightPanel.scrollTop;
        var lastLine = 1; // editor line starts with 1
        var lastScroll = 0;
        var nextLine = store.editor.state.doc.toString().split('\n').length; // number of lines of markdown
        var nextScroll = preview.offsetHeight - rightPanel.offsetHeight; // maximum scroll
        var lineMarkers = document.querySelectorAll("#".concat(store.uid, " .right-panel .preview > [data-sl]"));
        for (var i = 0; i < lineMarkers.length; i++) {
            var lineMarker = lineMarkers[i];
            if (lineMarker.offsetTop < scroll) {
                lastLine = parseInt(lineMarker.dataset.sl, 10);
                lastScroll = lineMarker.offsetTop;
            }
            else {
                nextLine = parseInt(lineMarker.dataset.sl, 10);
                nextScroll = lineMarker.offsetTop;
                break;
            }
        }
        var percentage = 0;
        if (lastScroll !== nextScroll) {
            percentage = (scroll - lastScroll) / (nextScroll - lastScroll);
        }
        // returns two neighboring marker lines, and current scroll percentage between two markers
        var r = {
            lastMarker: lastLine,
            nextMarker: nextLine,
            percentage: percentage,
        };
        return r;
    };
    var setEditorScroll = function (previewScroll) {
        var targetLineNumber = (previewScroll.nextMarker - previewScroll.lastMarker) *
            previewScroll.percentage +
            previewScroll.lastMarker;
        scrollEditor(targetLineNumber);
    };
    var syncPreview = (0, debounce_1.default)(function () {
        // sync right with left
        if (scrollingSide !== 'left') {
            var editorScroll = getEditorScroll();
            setPreviewScroll(editorScroll);
        }
    }, 128);
    var syncEditor = (0, debounce_1.default)(function () {
        // sync left with right
        if (scrollingSide !== 'right') {
            var previewScroll = getPreviewScroll();
            setEditorScroll(previewScroll);
        }
    }, 128);
    return { syncPreview: syncPreview, syncEditor: syncEditor };
};
exports.generateScrollMethods = generateScrollMethods;
//# sourceMappingURL=sync-scroll.js.map